#!/usr/local/bin/munger

; Copyright (c) 2005, 2006, James Bailie.
; All rights reserved.
; 
; Redistribution and use in source form, with or without modification, are
; permitted provided that the following conditions are met:  
; 
;     * Redistributions of source code must retain the above copyright
; notice, this list of conditions and the following disclaimer.
;     * The name of James Bailie may not be used to endorse or promote 
; products derived from this software without specific prior written permission.
; 
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS "AS IS"
; AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
; ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
; LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
; SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
; INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
; CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
; ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
; POSSIBILITY OF SUCH DAMAGE.

; -----------------------------------------------------------------------

; WEBLOG SCRIPT

; This script implements a weblog in Munger.  It uses both the FastCGI and
; SQLite interfaces of Munger.

; IT IS INTENDED FOR USE BY PROGRAMMERS, OR AT LEAST THOSE WHO UNDERSTAND 
; AND ARE COMFORTABLE USERS OF XHTML, CSS, SQL, AND HAVE SHELL ACCESS TO
; THE SYSTEMS WHERE THERE WEBLOGS RESIDE.  IF YOU LACK THESE SKILLS, YOU
; MUST USE DIFFERENT BLOG SOFTWARE.

; ------------------------------------------------------------------------

; INSTALLATION

; 1.  Place this script into a CGI-enabled directory, renaming it to have
;     a suffix recognized by the Apache FastCGI module.

; 2.  Edit the values of the eleven customization variables, described
;     further down, to reflect your system configuration.

; 3.  Ensure the path to the munger executable on the first (shebang) line
;     is correct for your system.

; 4.  Set the owner and permissions of the script accordingly.

; 5.  Fire up your browser and verify it works.

; ------------------------------------------------------------------------

; Creating Content:  Articles

; Articles are created with any text editor and then uploaded to the
; server, and fed to the incorporate.munger script, which adds the article
; to the database.  The incorporation script must be configured before use
; by altering the value of one variable at the top of the script.  There
; are no other administration facilities.  In particular, the script
; provides no facilities for editing or deleting posts or comments.  It is
; expected the blogger will use the sqlite3 command-line administration
; program to edit or delete articles and comments from the blog database.
; To do this requires knowledge of SQL and shell programming.

; The incorporation script does minimal filtering of the weblog posts.
; Groups of lines separated by blank lines are wrapped in <p></p> elements,
; and ' characters are escaped for SQLite, but no other processing is
; performed.  The blogger is therefore free to place XHTML 1.0 markup in
; his or her posts (to include images, for example).  The blogger must also
; know to escape both < and & with their corresponding character entities,
; &gt; and &amp; in order to use those characters outside of markup.

; Creating Content:  Styling

; The script offers no options to users to change the styling.  It is
; expected that the blogger will alter the sample CSS file to customize
; the styling.  The file is commented to aid modification.

; Creating Content:  Side Panel

; The optional content of the side panel, above the calendar, is supplied
; from a blogger-created data file which must contain markup suitable
; for inclusion in the panel.  The details of this mechanism is detailed
; below in the configuration note for the "panel" variable.

; Creating Content:  Comments

; If the "show_comments" variable is true, then users may comment on
; articles.  Comments are filtered to remove any markup elements users may
; place in them, and any remaining XHTML-significant characters are
; replaced with their corresponding character entities.

; Creating Content:  RSS

; Clicking a link above the calendar causes the script to generate a RSS
; 0.91 summary of the articles which appear on the front page of the blog.

; Administration

; There are two tables in the database file, one named Articles and one
; named Comments.  The structure of these tables can be ascertained with
; the sqlite3 .schema command.  Suffice it here to say the Articles table
; is keyed on the Id column, the content of which is a unique identifier
; for each record.  Each record in the Comments table also has an Id
; column, whose content is the Id in the Articles table of the article each
; comment is associated with.

; I will describe an example of using sqlite3 to make a modification to the
; database.  This is only a description that will make sense to those who
; understand shell and SQL programming, rather than an explicit,
; step-by-step example.  If this description is gobbledy-gook to you, you
; should not use this script.  That statement is not meant to be
; derogatory.  Use the tool that is comfortable for your skillset, and you
; will be much happier.

; Let's say you wish to edit a post you made, to correct a spelling error.
; You run sqlite3 and invoke a SQL SELECT statement using the LIKE clause
; to specify a unique phrase in the desired article's title, requesting the
; Body and Id columns.  When the program spits out its reponse, you verify
; that only one article has been selected and it is the one you wish to
; modify.  Now you tell sqlite3 to send its output to a file with the
; .output command, and use the program's command-line history feature to
; repeat the SELECT statement.  Then you exit sqlite3 and edit the output
; file in a text editor, correcting your mistake, and ensuring that any new
; paragraphs you create are wrapped in their own <p></p> elements, and that
; you have escaped any ' characters by doubling them, like so ''.  Now you
; change the output file into a shell script which invokes sqlite3 on the
; database file, with its standard input redirected to a "here" document.
; You create a "here" document containing a SQL UPDATE statement, which
; SETs the Body column to the edited body data, and uses the Id value to
; specify the article to be affected in a WHERE clause.  The sqlite3
; program expects commands to be terminated with semi-colons.  This
; conveniently allows one to spread commands and input data over multiple
; lines.  You run the shell script, and database is updated.

; IF YOU REMOVE COMMENTS FROM THE COMMENTS TABLE, YOU NEED TO DECREMENT THE
; VALUE OF THE COMMENTS FIELD OF THE ASSOCIATED ARTICLE, IF YOU WANT TO
; KEEP IT ACCURATE.  It is the count of the number of comments which are
; associated with the article.  To modify a comment, one must find a
; unique phrase in the Author or Body fields to identify it in SQL
; statements, since the ID value is not unique.

; -----------------------------------------------------------------------

; THE FOLLOWING ELEVEN VARIABLES NEED TO BE SET TO REFLECT YOUR CONFIGURATION.

; "stylesheet" is the fully-qualified filename of the stylesheet for the
; generated pages.  Here the root directory is the DocumentRoot defined
; httpd.conf.

(setq stylesheet "/blog.css")

; "title" is the title for the Blog and appears at the top of the panel as
; an <h1></h1> element.

(setq title "Tempermental Musings")

; "path" is the path to the location of this script, as it needs to be
; specified in URIs, and "host" is the hostname.  For example, for this
; URI, http://127.0.0.1/cgi-bin/blog.fcgi, the path would be, /cgi-bin/ and
; the host would be, 127.0.0.1

; The path must begin and end with a slash.  The hostname must be the bare
; hostname.

(setq path "/cgi-bin/")
(setq host "127.0.0.1")

; "db_path" is the fully-qualified directory specifying where the blog's
; database will reside.  The value must have a trailing slash.  It must be
; writable by user www or whatever user your server runs as, so the script
; can create the database file if it does not exist.

(setq db_path "/usr/local/www/db/")

; "db_name" is the just the filename, sans any path information, of the
; SQLite database file to hold the database.  The databse will be created
; and initialized if it does not exist.

(setq db_name "blog.db")

; "before" and "after" are the fully-qualified filenames of two files
; containing material for the panel, which will appear under the title in
; the panel sidebar.  The files may contain anything you wish, including
; markup.  The file "before" will be inserted before the navigation items,
; and the file "after" will be inserted after the navigation items.  You
; may set either variable to the empty string, if you do not wish to add
; content in that position of the panel.

(setq before (concat db_path "before.txt"))
(setq after (concat db_path "after.txt"))

; "page_size" is the number of articles or comments displayed on one
; page.  If more posts or comments are generated by a query of the 
; database, links will appear at the bottom of the listing allowing
; the user to page through the material page_size items at a time.
; If you intend to write long articles, you may wish to set page_size
; to 1.

(setq page_size 10)

; "comment_page_size" limits the number of items displayed on the comment
; pages for an article.  Since comments are typically shorter than articles
; you may wish to set comments to a different value than page_size.  If you
; have set page_size to 1, for example, it would be a bad idea to set
; comment_page_size to 1.

(setq comment_page_size 10)

; "show_comments" is a boolean which turns comments on or off.

(setq show_comments 1)

; "name" is the basename of this script as it appears in URIs.  Invoking
; (concat "http://" host path name) should create the toplevel URI for the
; script.  This script is distributed as blog.txt and not blog.fcgi to stop
; Apache from attempting to run it, instead of sending it the client.
; Whatever name you choose to give the file, it must have a suffix
; recognized by the FastCGI module.

(setq name "blog.fcgi")

; "rss_description" is the blurb which will appear in the <description>
; child element of the <channel> element for the blog's RSS feed.  It
; should be a concise description of the blog.

(setq rss_description "Your description goes here.")

; Nothing below this line needs to be altered.

; -------------------------------------------------------------------

(fatal)

(let ((ver (version)))
   (unless (and (>= (car ver) 4) (>= (cadr ver) 83))
      (die "This script requires Munger 4.83 or better.")))

(load (join "/" (libdir) "fcgi.munger"))

; Used to prevent SQL injection attacks.

(let ((sql_rx (regcomp "'")))
   (defun sql_clean (t)
      (substitute sql_rx "''" t 0)))

; Utility function to sanitize comments and RSS feed.

(setq apos_rx (regcomp "'"))

(let ((amp_rx (regcomp "&"))
      (markup_rx (regcomp "</?[^>]+/?>")))

   (defun remove_markup (txt)
      (substitute apos_rx "&#39;" 
         (substitute amp_rx "&amp;"
            (substitute markup_rx "" txt 0) 0) 0)))

; Creates and/or opens the database.  The database handle
; is bound to db.

(let ((db_file (concat db_path db_name)))

   (setq result (exists db_file))

   (when (stringp (setq db (sql_open db_file)))
      (die "sql_open: " db))
   
   (unless result
      (setq sql 
         (concat "CREATE TABLE Articles ( Id TEXT PRIMARY KEY UNIQUE, Date TEXT, "
            "Title TEXT, Body TEXT, Time INTEGER, Comments INTEGER )"))

      (when (stringp (setq result (sql_exec db sql)))
         (die "sql_exec: " result))
   
      (setq sql 
         (concat "CREATE TABLE Comments ( Id TEXT, Date TEXT, "
            "Author TEXT, Body TEXT, Time INTEGER, Dummy INTEGER )"))

      (when (stringp (setq result (sql_exec db sql)))
         (die "sql_exec: " result))

      (chown "www" "www" db_file)
      (chmod "600" db_file)))
   
(defun send_day_wrapper ()
   (main send_day))

(defun send_recent_wrapper ()
   (main send_recent))

(defun send_month_wrapper ()
   (main send_month))

(defun send_one_wrapper ()
   (main send_one))

(defun send_comments_wrapper ()
   (main send_comments))

(defun main (func)
   (send_headers title stylesheet "" "" "" "")

   (cgi_println "<div id=\"container\">")
   (cgi_println "<div id=\"background\">")
   (cgi_println "<div id=\"main\">")

   (func)

   (cgi_println "</div>")
   (cgi_println "</div>")

   (cgi_println "<div id=\"panel\">")

   (send_panel)

   (cgi_println "</div>")
   (cgi_println "</div>")

   (cgi_println "</body>")
   (cgi_println "</html>"))

(let ((line ""))

   (defun send_panel ()
      (cgi_println "<h1>" title "</h1>")
   
      (when before (cgi_sendfile before))

      (cgi_println "<div class=\"center\">")

      (cgi_println "<h4>Navigation</h4>")

      (cgi_println "<a href=\"" path name "\">Recent Articles</a><br/>")
      (cgi_println "<a href=\"" path name "?request=send_rss\">RSS Feed</a>")

      (cgi_println "</div>")

      (cgi_println "<div id=\"search\">")

      (cgi_println "<form action=\"" path name "\" method=\"post\" >")
      (cgi_println "<label for=\"key\">Search the article database:</label><br/>")
      (cgi_println "<input id=\"searchtext\" type=\"text\" name=\"key\" value=\"\" />")
      (cgi_println "<input type=\"hidden\" name=\"request\" value=\"send_search\" />")
      (cgi_println "</form>")

      (cgi_println "</div>")

      (print_month (lookup data "year") (lookup data "mon"))

      (when after (cgi_sendfile after))))
   
(let ((sql "")
      (result ()))

   (defun send_rss ()
      (setq sql (stringify "SELECT Title,Id,Body FROM Articles ORDER BY Time DESC LIMIT " page_size))
      (setq result (sql_exec db sql))

      (when (stringp result)
         (die "sql_exec: " result))

      (cgi_println "Content-Type: text/xml")
      (cgi_newline)

      (cgi_println "<?xml version=\"1.0\" encoding=\"utf-8\"?>")
      (cgi_println "<rss version=\"0.91\">")
      (cgi_println "<channel>")
      (cgi_println "<title>" (remove_markup title) "</title>")
      (cgi_println "<link>http://" host path name "</link>")
      (cgi_println "<description>" (remove_markup rss_description) "</description>")

      (when result
         (setq result (cdr result))
   
         (while result
            (cgi_println "<item>")
            (cgi_println "<title>" (remove_markup (caar result)) "</title>")
            (cgi_println "<link>http://" host path name "?request=send_one&amp;id=" (cadr (car result)) "</link>")
            (cgi_println "<description>" (concat (remove_markup (substring (caddr (car result)) 0 160)) "...")
               "</description>")
            (cgi_println "</item>")
            (setq result (cdr result))))
   
      (cgi_println "</channel>")
      (cgi_println "</rss>")))

(let ((local ())
      (mnth "")
      (first_day 0)
      (last_day 0)
      (first_sunday 0)
      (last_saturday 0))

   (defun print_month (year mon)

      ; Determine the year and month.

      (when year (setq year (digitize year)))
      (when mon (setq mon (digitize mon)))
      (setq local (localtime (time)))

      (cond ((and (not year) (not mon))
             (setq year (car local))
             (setq mon (cadr local)))

            ((and (not year) mon)
             (setq year (car local)))

            ((and year (not mon))
             (setq mon (cadr local))))

      (setq mnth (month mon))

      ; Find first day of month.

      (setq first_day (date2days year mon 1))

      ; Find last day of month.
      
      (if (eq 12 mon)
         (setq last_day (date2days (+ 1 year) 1 1))
         (setq last_day (date2days year (+ 1 mon) 1)))
         
      (dec last_day)

      ; Find first Sunday of calendar.  May be part of previous month.
   
      (setq first_sunday first_day)
      
      (while (car (weekday first_sunday))
         (dec first_sunday))

      ; Find last Saturday of calendar.  May be last day of month or in
      ; next month.

      (setq last_saturday last_day)

      (while (not (eq 6 (car (weekday last_saturday))))
         (inc last_saturday))

      (cgi_println "<div id=\"caldiv\">")

      (print_calendar_title year mon mnth (car local) (cadr local))
      (print_calendar first_day last_day first_sunday last_saturday)

      (cgi_println "</div>")))

(let ((prev_year 0)
      (next_year 0)
      (prev_mon 0)
      (next_mon 0))

   (defun print_calendar_title (year mon mnth real_year real_mon)

      (if (not (eq mon 1))
         (progn
            (setq prev_year year)
            (setq prev_mon (- mon 1)))
   
         (setq prev_year (- year 1))
         (setq prev_mon 12))
         
      (if (not (eq mon 12))
         (progn
            (setq next_mon (+ mon 1))
            (setq next_year year))

         (setq next_year (+ year 1))
         (setq next_mon 1))
         
      (cgi_println "<table id=\"cal_title\">")

      (cgi_println "<tr><td id=\"cal_title_left\">"
         (if (> prev_year 1969)
            (stringify "<a href=\"" path name "?request=send_month"
                                              "&amp;time=" (date2time prev_year prev_mon 1)
                                              "&amp;year=" prev_year "&amp;mon=" prev_mon "\">&lt;-</a></td>")
            "&lt;")
   
         "<td id=\"cal_title_center\"><a href=\"" path name "?request=send_month"
                                                  "&amp;time=" (date2time year mon 1)
                                                  "&amp;year=" year "&amp;mon=" mon "\">" mnth " " year "</a></td>"

         "<td id=\"cal_title_right\">"
         "<a href=\"" path name "?request=send_month"
                                "&amp;time=" (date2time next_year next_mon 1)
                                "&amp;year=" next_year "&amp;mon=" next_mon "\">-&gt;</a></td></tr>")

      (cgi_println "</table>")))

(let ((dom ())
      (sql "")
      (t 0)
      (t2 0)
      (result ()))
   
   (defun print_calendar (first_day last_day first_sunday last_saturday)
      
      ; Print the calendar.
      
      (cgi_println "<table id=\"calendar\">")
      (cgi_println "<thead><tr><td>Sun</td><td>Mon</td><td>Tue</td><td>Wed</td>"
         "<td>Thu</td><td>Fri</td><td>Sat</td></tr></thead>")
      (cgi_print "<tr>")
      
      (for (day first_sunday last_saturday)
         (cgi_print "<td>")
   
         (when (and (>= day first_day) (<= day last_day))
            (setq dom (days2date day))
            (setq sql (concat "SELECT Count(*) FROM Articles WHERE Time >= "
                              (setq t (date2time (car dom) (cadr dom) (caddr dom)))
                              " AND Time < " (eval (cons 'date2time (days2date (+ day 1))))))
            (setq result (sql_exec db sql))

            (when (stringp result)
               (die "sql_exec: " result))

            (if (digitize (caar (cdr result)))
               (cgi_print "<a href=\"" path name "?request=send_day&amp;time=" t 
                  "&amp;year=" (car dom) "&amp;mon=" (cadr dom) "\">" (caddr dom) "</a>")
               (cgi_print (caddr dom))))

         (cgi_print "</td>")

         (when (and (< day last_saturday) 
                    (eq 6 (car (weekday day))))
            (cgi_println "</tr>")
            (cgi_print "<tr>")))

      (cgi_println "</tr>")
      (cgi_println "</table>")))

(let ((second 0)
      (n 0))

   (defun print_each (request result count comments)
      (setq second 0)

      (while result
         (inc count)
         (inc second)

         (when (and comments (cadr comments) (eq 2 second))
            (cgi_println "<div class=\"comtitle\">")
            (cgi_println "<h2>Comments</h2>")
            (cgi_println "</div>"))
   
         (cgi_println "<div class=\"article\">")
   
         (cgi_println "<div class=\"date\">")
         (cgi_println (caar result))
         (cgi_println "</div>")
   
         (cgi_println "<div class=\"internal\">")

         (if (and comments (> second 1))
            (cgi_println "<h3>" (cadr (car result)) " comments:</h3>")
            (cgi_println "<h3>" (cadr (car result)) "</h3>"))

         (cgi_println (caddr (car result)))
         (cgi_println "</div>")

         (cgi_println "<div class=\"link\">")

         (if (and show_comments (not comments))
            (progn
               (setq n (digitize (caddddr (car result))))
      
               (cgi_println "<a href=\"" path name "?request=send_comments&amp;id=" (cadddr (car result)) 
                  "&amp;old_request=" (encode request) "&amp;old_index=" (- count 1) "\" rel=\"bookmark\">"
                  n " comment" (if (not (eq n 1)) "s" "") "</a>"))

            (if (and show_comments (> count 1))
               (cgi_println "Comment " (- count 1))
               (cgi_println "&nbsp;")))

         (cgi_println "</div>")
         (cgi_println "</div>")
 
         (setq result (cdr result)))

      count))

(let ((tmp 0))
   (defun print_nav (request old_count count comments)
   
      (if (not (or old_count 
                (and (not old_count) comments)
                (and count (not (% count (if comments comment_page_size page_size))))))
   
         (when comments
            (print_comment_form comments (or count old_count)))
   
         (cgi_println "<div class=\"pager\">")
   
         (when (and (not old_count) comments)
            (cgi_print "<a href=\"" path name "?request=" (decode (caddr comments))
                       "&amp;index=" (cadddr comments) "\">[ Back to Article Listing ]</a>&nbsp;&nbsp;"))
   
         (when old_count 
            (when (< (setq tmp (- old_count (if comments comment_page_size page_size))) 0)
               (setq tmp 0))
            
            (cgi_print "<a href=\"" path name "?request=" request)
   
            (when comments
               (cgi_print "&amp;old_request=" (caddr comments)
                          "&amp;old_index=" (cadddr comments)))
   
            (cgi_print "&amp;index=" tmp "\">[ Previous ]</a>&nbsp;&nbsp;"))
               
         (if (not (and count (not (% count (if comments comment_page_size page_size)))))
            (progn
               (cgi_println "</div>")

               (when comments
                  (print_comment_form comments (or count old_count))))
      
            (cgi_print "<a href=\"" path name "?request=" request)
   
            (when comments
               (cgi_print "&amp;old_request=" (caddr comments)
                          "&amp;old_index=" (cadddr comments)))
   
            (cgi_println "&amp;index=" count "\">[ Next ]</a>")
            (cgi_println "</div>")))))
   
(defun print_results (title result request old_count count (id))
   (cgi_println "<div class=\"comtitle\">")
   (cgi_println "<h2>" title "</h2>")
   (cgi_println "</div>")

   (cond ((stringp result)
          (die "sql_exec: " result))

         ((not result)
          (cgi_println "<div class=\"pager\"><span class=\"bold\">No " (if count "more " " ")
             (if id "comments" "articles") " found.</span></div>")

          (when old_count
            (setq count 0)
            (print_nav request old_count count id)))

          (1 (setq result (cdr result))
             (print_nav request old_count (print_each request result count id) id))))

(defun print_comment_form (id count)
   (cgi_println "<div class=\"comtitle\">")
   (cgi_println "<h2>Add A Comment</h2>")

   (cgi_println "</div>")
   (cgi_println "<div class=\"article\">")

   (cgi_println "<div class=\"date\">&nbsp;</div>")
   (cgi_println "<div class=\"internal\">")

   (cgi_println "<h3>Both fields must be non-empty.</h3>")
   (cgi_println "<form action=\"" path name "\" method=\"post\">")

   (cgi_println "<p><input type=\"hidden\" name=\"request\" value=\"add_comment\" />")
   (cgi_println "<input type=\"hidden\" name=\"id\" value=\"" (car id) "\" />")

   (cgi_println "<input type=\"hidden\" name=\"index\" value=\"" count "\" />")
   (cgi_println "<input type=\"hidden\" name=\"old_request\" value=\"" (caddr id) "\" />")

   (cgi_println "<input type=\"hidden\" name=\"old_index\" value=\"" (cadddr id) "\" />")
   (cgi_println "<label for=\"author\">Your name/nickname:</label><br/>")

   (cgi_println "<input id=\"comauth\" type=\"text\" name=\"author\" value=\"\" /><br/>")
   (cgi_println "<label for=\"body\">Your comment:</label><br/>")

   (cgi_println "<textarea id=\"combody\" name=\"body\" rows=\"10\" ></textarea><br/>")
   (cgi_println "</p><p><input type=\"submit\" value=\"Submit\" /></p>")

   (cgi_println "</form>")
   (cgi_println "</div>")

   (cgi_println "<div class=\"link\">&nbsp;</div>")
   (cgi_println "</div>"))

(defun send_search_wrapper ()
   (if (lookup data "key")
      (main send_search)
      (send_recent_wrapper)))

(let ((count 0)
      (old_count 0)
      (sql "")
      (t 0)
      (mnth "")
      (nd 0)
      (blank_rx (regcomp (concat "[\b\t]*" (char 10) (char 10))))
      (term_rx (regcomp (concat (char 13) (char 10))))
      (index 0)
      (result ()))
   
   (defun add_comment ()
      (setq count (digitize (or (lookup data "index") "0")))
      (setq nd (sql_clean (lookup data "author")))
      (setq mnth (sql_clean (lookup data "body")))
      (setq t (sql_clean (lookup data "id")))

      (when nd mnth
         (setq nd (remove_markup nd))

         (setq mnth
            (concat "<p>" 
                    (substitute blank_rx 
                                "</p><p>" 
                                (substitute term_rx (char 10) (remove_markup mnth) 0)
                                0)
                    "</p>"))

         (setq sql (concat "SELECT Comments FROM Articles WHERE Id = '" t "'"))
         (setq result (sql_exec db sql))
      
         (when (stringp result)
            (die "sql_exec: " result))

         (setq index (+ 1 (digitize (caar (cdr result)))))
         (setq sql (stringify "UPDATE Articles SET Comments = " index " WHERE Id = '" t "'"))
         (setq result (sql_exec db sql))
      
         (when (stringp result)
            (die "sql_exec: " result))

         (setq sql (concat "INSERT INTO Comments VALUES ('" t "','" (date) "','" 
               nd "','" mnth "'," (time) ",0)"))
         (setq result (sql_exec db sql))

         (when (stringp result)
            (die "sql_exec: " result)))

      (cgi_println "Content-Type: text/html")
      (cgi_println "Location: http://" host path name "?request=send_comments&id=" t "&index=" 
         (- count (% count comment_page_size))
         "&old_request=" (lookup data "old_request") "&old_index=" (lookup data "old_index"))
      (cgi_newline))

   (defun send_search ()
      (setq old_count (setq count (digitize (or (lookup data "index") "0"))))
      (setq t (sql_clean (lookup data "key")))

      (setq sql (stringify "SELECT Date,Title,Body,Id,Comments FROM Articles WHERE "
                           "Title LIKE '%" (substitute apos_rx "''" t 0) 
                           "%' OR Body LIKE '%" (substitute apos_rx "''" t 0) "%' ORDER BY Time DESC"
                           " LIMIT " page_size " OFFSET " count))
      (setq result (sql_exec db sql))

      (print_results (concat "Articles Containing: " t) 
                     result 
                     (concat "send_search&amp;key=" (encode t))
                     old_count
                     count))

   (defun send_comments ()
      (setq old_count (setq count (digitize (or (lookup data "index") "0"))))
      (setq t (sql_clean (lookup data "id")))
      (setq nd (sql_clean (lookup data "old_request")))
      (setq mnth (sql_clean (lookup data "old_index")))

      (setq sql (stringify "SELECT Date,Title,Body,Id,Comments,Time FROM Articles WHERE Id = '" t "' "
                           "UNION ALL "
                           "SELECT Date,Author,Body,Id,Dummy,Time FROM Comments WHERE Id = '" t "' ORDER BY Time "
                           "LIMIT " comment_page_size " OFFSET " count))
      (setq result (sql_exec db sql))

      (print_results (if count "Comments" "Article")
                     result
                     (concat "send_comments&amp;id=" t)
                     old_count count t (if count 0 1) (encode nd) mnth))

   (defun send_one ()
      (setq nd (sql_clean (lookup data "id")))
      (setq sql (concat "SELECT Date,Title,Body,Id,Comments FROM Articles WHERE Id = '" nd "'"))
      (setq result (sql_exec db sql))

      (print_results (if result (cadr (cadr result)) "Not Found") 
                     result 
                     (concat "send_one&amp;id=" nd)
                     0
                     0))

   (defun send_month ()
      (setq old_count (setq count (digitize (or (lookup data "index") "0"))))
      (setq t (sql_clean (lookup data "time")))
      (setq nd (localtime t))
      (setq mnth (stringify "Articles Posted In " (month (cadr nd)) " " (car nd)))

      (if (eq (cadr nd) 12)
         (setq nd (date2time (+ 1 (car nd)) 1 1))
         (setq nd (date2time (car nd) (+ 1 (cadr nd)) 1)))

      (setq sql (stringify "SELECT Date,Title,Body,Id,Comments FROM Articles WHERE Time >= " t
                           " AND Time < " nd
                           " ORDER BY Time DESC LIMIT "
                           page_size " OFFSET " count))
      (setq result (sql_exec db sql))

      (print_results mnth
                     result
                     (concat "send_month&amp;time=" t "&amp;year=" (or (lookup data "year") "0")
                             "&amp;mon=" (or (lookup data "mon") "0"))
                     old_count count))

   (defun send_recent ()
      (setq old_count (setq count (digitize (or (lookup data "index") "0"))))
      (setq sql (stringify "SELECT Date,Title,Body,Id,Comments FROM Articles ORDER BY Time DESC LIMIT " 
                           page_size " OFFSET " count))
      (setq result (sql_exec db sql))
   
      (print_results "Recent Articles" result "send_recent" old_count count))

   (defun send_day ()
      (setq old_count (setq count (digitize (or (lookup data "index") "0"))))
      (setq t (lookup data "time"))

      (setq nd (localtime t))
      (setq mnth (stringify "Articles Posted On " (caddr nd) " " (month (cadr nd)) " " (car nd)))
      (setq nd (date2days (car nd) (cadr nd) (caddr nd)))
      (setq nd (days2date (+ nd 1)))
      (setq nd (date2time (car nd) (cadr nd) (caddr nd)))
   
      (setq sql (stringify "SELECT Date,Title,Body,Id,Comments FROM Articles WHERE Time >= " t 
                           " AND Time < " nd
                           " ORDER By Time DESC LIMIT "
                           page_size " OFFSET " count))
      (setq result (sql_exec db sql))
   
      (print_results mnth
                     result 
                     (concat "send_day&amp;time=" t "&amp;year=" (or (lookup data "year") "0")
                             "&amp;mon=" (or (lookup data "mon") "0"))
                     old_count count)))

(defun service_request ()
   (get_data)
   (setq request (lookup data "request"))

   (cond ((not request)
          (main send_recent))

         ((setq func (and request (lookup requests request)))
          (func))

         (1 (main send_recent))))

; Establish request handlers.

(setq requests (table))

(hash requests "send_recent" send_recent_wrapper)
(hash requests "send_day" send_day_wrapper)
(hash requests "send_month" send_month_wrapper)

(hash requests "send_comments" send_comments_wrapper)
(hash requests "send_one" send_one_wrapper)
(hash requests "send_search" send_search_wrapper)

(hash requests "add_comment" add_comment)
(hash requests "send_rss" send_rss)

; Main loop.

(while (cgi_accept)
   (service_request))

(sql_close db)

(quit)
